/*
 * Copyright (C) 2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#include "platforms.h"

namespace AOT_BMG {
constexpr AOT::PRODUCT_CONFIG productConfigs[] = {AOT::BMG_G21_A0};
}
